package com.songoda.skyblock.utils.item;

import com.songoda.skyblock.SkyBlock;
import com.songoda.skyblock.placeholder.Placeholder;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryCloseEvent;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.*;
import java.util.Map.Entry;

public class nInventoryUtil {

    private final Player player;
    private Listener listener;

    private Inventory inv;

    private String title;
    private int size = 9;
    private InventoryType type;
    private final Map<Integer, ItemStack> items = new HashMap<>();

    public nInventoryUtil(Player player, final ClickEventHandler handler) {
        this.player = player;

        if (handler != null) {
            this.listener = new Listener() {
                @EventHandler
                public void onInventoryClick(InventoryClickEvent event) {
                    if (!(event.getWhoClicked() instanceof Player)) {
                        return;
                    }

                    if (inv != null && event.getInventory().equals(inv)) {
                        event.setCancelled(true);

                        if (event.getCurrentItem() == null || event.getCurrentItem().getType() == Material.AIR) {
                            return;
                        }

                        ClickEvent clickEvent = new ClickEvent(event.getClick(), event.getSlot(), event.getCurrentItem());
                        handler.onClick(clickEvent);

                        if (!clickEvent.getCancelled()) {
                            event.setCancelled(false);
                        }

                        if (clickEvent.getWillClose()) {
                            event.getWhoClicked().closeInventory();
                        }

                        if (clickEvent.getWillDestroy()) {
                            destroy();
                        }
                    }
                }

                @EventHandler
                public void onInventoryClose(InventoryCloseEvent event) {
                    if (!(event.getPlayer() instanceof Player)) {
                        return;
                    }

                    Inventory inv = event.getInventory();

                    if (inv.equals(nInventoryUtil.this.inv)) {
                        inv.clear();
                        destroy();
                    }
                }

                @EventHandler
                public void onPlayerQuit(PlayerQuitEvent event) {
                    if (event.getPlayer().getUniqueId().equals(player.getUniqueId())) {
                        destroy();
                    }
                }
            };

            Bukkit.getPluginManager().registerEvents(listener, SkyBlock.getInstance());
        }
    }

    public void addItem(Item item, int... slots) {
        Arrays.stream(slots).forEachOrdered(slotList -> items.put(slotList, item.prepareItem()));
    }

    public void addItemStack(ItemStack is, int... slots) {
        Arrays.stream(slots).forEachOrdered(slotList -> items.put(slotList, is));
    }

    public Map<Integer, ItemStack> getItems() {
        return items;
    }

    public Item createItem(ItemStack is, String itemDisplayname, List<String> itemLore, Placeholder[] placeholders, Enchantment[] itemEnchantments, ItemFlag[] itemFlags) {
        return new Item(is, itemDisplayname, itemLore, placeholders, itemEnchantments, itemFlags);
    }

    public void open() {
        createInventory();
        player.openInventory(inv);
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setType(InventoryType type) {
        this.type = type;
    }

    public void setRows(int rows) {
        if (rows > 6 || rows < 0) {
            size = 9;
            return;
        }

        this.size = rows * 9;
    }

    public void setSize(int size) {
        this.size = size;
    }

    public void createInventory() {
        if (type == null) {
            if (title == null) {
                inv = Bukkit.createInventory(null, size);
            } else {
                inv = Bukkit.createInventory(null, size, title);
            }
        } else {
            if (title == null) {
                inv = Bukkit.createInventory(null, type);
            } else {
                inv = Bukkit.createInventory(null, type, title);
            }
        }

        items.forEach((key, value) -> inv.setItem(key, value));
    }

    public Inventory getInventory() {
        return inv;
    }

    public void setInventory(Inventory inv) {
        this.inv = inv;
    }

    public void destroy() {
        if (listener != null) {
            HandlerList.unregisterAll(listener);
        }

        title = null;
        type = null;
        inv = null;
        items.clear();

        listener = null;
    }

    public interface ClickEventHandler {
        void onClick(ClickEvent event);
    }

    public static class Item {

        private final ItemStack is;
        private final String itemDisplayname;
        private List<String> itemLore;
        private final Placeholder[] placeholders;
        private final Enchantment[] itemEnchantments;
        private final ItemFlag[] itemFlags;

        public Item(ItemStack is, String itemDisplayname, List<String> itemLore, Placeholder[] placeholders, Enchantment[] itemEnchantments, ItemFlag[] itemFlags) {
            this.is = is;
            this.itemDisplayname = itemDisplayname == null ? null : ChatColor.translateAlternateColorCodes('&', itemDisplayname);
            this.itemLore = itemLore;
            this.placeholders = placeholders;
            this.itemEnchantments = itemEnchantments;
            this.itemFlags = itemFlags;
        }

        public void setLore() {
            if (itemLore != null) {
                List<String> formattedItemLore = new ArrayList<>(itemLore.size());

                for (String itemLoreList : itemLore) {
                    if (placeholders != null) {
                        for (Placeholder placeholderList : placeholders) {
                            if (itemLoreList.contains(placeholderList.getPlaceholder())) {
                                itemLoreList = ChatColor.translateAlternateColorCodes('&', itemLoreList.replace(placeholderList.getPlaceholder(), placeholderList.getResult()));
                            }
                        }
                    }

                    formattedItemLore.add(ChatColor.translateAlternateColorCodes('&', itemLoreList));
                }

                itemLore.clear();
                itemLore = formattedItemLore;
            }
        }

        public void setItemMeta() {
            ItemMeta im = is.hasItemMeta() ? is.getItemMeta() : Bukkit.getItemFactory().getItemMeta(is.getType());
            im.setDisplayName(itemDisplayname);
            im.setLore(itemLore);

            if (itemFlags != null) {
                im.addItemFlags(itemFlags);
            }

            if (itemEnchantments != null) {
                for (Enchantment itemEnchantmentList : itemEnchantments) {
                    im.addEnchant(itemEnchantmentList, 1, true);
                }
            }

            is.setItemMeta(im);
        }

        public ItemStack prepareItem() {
            setLore();
            setItemMeta();

            return is;
        }
    }

    public class ClickEvent {

        private final ClickType click;
        private final int slot;
        private final ItemStack is;

        private boolean close = true;
        private boolean destroy = true;
        private boolean cancelled = true;

        public ClickEvent(ClickType click, int slot, ItemStack is) {
            this.click = click;
            this.slot = slot;
            this.is = is;
        }

        public ClickType getClick() {
            return click;
        }

        public int getSlot() {
            return slot;
        }

        public ItemStack getItem() {
            return is;
        }

        public boolean getWillClose() {
            return close;
        }

        public void setWillClose(boolean close) {
            this.close = close;
        }

        public boolean getWillDestroy() {
            return destroy;
        }

        public void setWillDestroy(boolean destroy) {
            this.destroy = destroy;
        }

        public boolean getCancelled() {
            return cancelled;
        }

        public void setCancelled(boolean cancelled) {
            this.cancelled = cancelled;
        }
    }
}
